/*
 * Copyright (c) 2014 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*global document, window, tizen, console*/

/**
 * App implementation.
 * @namespace Compass/main
 * @memberof Compass
 */
(function strict() {
    'use strict';

    // application data:
    var directionEl = null,
        angleEl = null,
        rotationEl = null,

        orientationData = null,
        orientationCheckInterval = null,
        orientationCheckAttemptsLeft = 10,
        rotation = 0;   // compass rotation

    /**
     * Obtains angle from rotation.
     * @param {number} value
     * @return {number}
     */
    function angleFromRotation(value) {
        var angle = -value % 360;

        if (angle < 0) {
            angle += 360;
        }
        return angle;
    }

    /**
     * Handles device orientation data.
     * @memberof Compass/main
     * @param {DeviceOrientationEvent} eventData Device orientation event.
     */
    function onDeviceOrientation(eventData) {

        var angle = eventData.alpha,
            lastAngle = angleFromRotation(rotation),
            deltaAngle = 0,
            text = '';

        orientationData = eventData;

        // Check if the angle has changed since the last measurement.
        if (isNaN(angle) || angle === lastAngle) {
            return;
        }

        // find direction from given angle
        if (angle < 68 || angle > 292) {
            text = 'N';
        } else if (angle > 112 && angle < 248) {
            text = 'S';
        }
        if (angle > 22 && angle < 158) {
            text += 'E';
        } else if (angle > 202 && angle < 338) {
            text += 'W';
        }

        // Calculate the rotation of the compass.
        deltaAngle = lastAngle - angle;
        if (deltaAngle > 180) {
            rotation -= 360 - deltaAngle;
        } else if (deltaAngle < -180) {
            rotation += 360 + deltaAngle;
        } else {
            rotation += deltaAngle;
        }

        // Update the direction and angle labels.
        directionEl.innerHTML = text;
        angleEl.innerHTML =
            Math.round(angle) + '<sup>o</sup>';
        // Rotate the compass image.
        rotationEl.style['-webkit-transform'] =
            'rotate(' + rotation + 'deg)';
    }

    /**
     * Exits the application.
     * @memberof Compass/main
     */
    function exit() {
        try {
            tizen.application.getCurrentApplication().exit();
        } catch (err) {
            console.error('Error: ', err);
        }
    }

    /**
     * Handles no orientation support case.
     * @memberof Compass/main
     */
    function onOrientationNotSupported() {
        window.alert('Device orientation is not supported.');
        exit();
    }

    /**
     * Checks if orientation is supported by the device.
     * @memberof Compass/main
     */
    function checkOrientationSupport() {
        orientationCheckInterval = window.setInterval(
            function checkOrientationSupport() {
                if (orientationData !== null &&
                    orientationData.alpha !== null) {
                    window.clearInterval(orientationCheckInterval);
                } else {
                    orientationCheckAttemptsLeft -= 1;
                    if (orientationCheckAttemptsLeft === 0) {
                        window.clearInterval(orientationCheckInterval);
                        onOrientationNotSupported();
                    }
                }
            }, 200);
    }

    /**
     * Binds events.
     * @memberof Compass/main
     */
    function bindEvents() {
        window.addEventListener('tizenhwkey', function onTizenHWKey(e) {
            if (e.keyName === 'back') {
                exit();
            }
        });
        window.addEventListener(
            'deviceorientation',
            onDeviceOrientation,
            false
        );
    }

    /**
     * Initializes the application.
     * @memberof Compass/main
     */
    function init() {
        directionEl = document.getElementById('direction');
        angleEl = document.getElementById('angle');
        rotationEl = document.getElementById('rotation');

        bindEvents();
        checkOrientationSupport();
    }

    // initialize application when the document is ready
    window.addEventListener('DOMContentLoaded', init);

}());
